@@ -7,6 +7,7 @@ from .enums import (
     MethodReturnValue,
     TableBordersLayout,
     TableCellFillMode,
+    TableHeadingsDisplay,
     WrapMode,
     VAlign,
     TableSpan,
@@ -47,6 +48,7 @@ class Table:
         padding=None,
         outer_border_width=None,
         num_heading_rows=1,
+        repeat_headings=1,
     ):
         """
         Args:
@@ -79,6 +81,7 @@ class Table:
             num_heading_rows (number): optional. Sets the number of heading rows, default value is 1. If this value is not 1,
                 first_row_as_headings needs to be True if num_heading_rows>1 and False if num_heading_rows=0. For backwards compatibility,
                 first_row_as_headings is used in case num_heading_rows is 1.
+            repeat_headings (fpdf.enums.TableHeadingsDisplay): optional, indicates whether to print table headings on every page, default to 1.
         """
         self._fpdf = fpdf
         self._align = align
@@ -98,6 +101,7 @@ class Table:
         self._width = fpdf.epw if width is None else width
         self._wrapmode = wrapmode
         self._num_heading_rows = num_heading_rows
+        self._repeat_headings = TableHeadingsDisplay.coerce(repeat_headings)
         self._initial_style = None
         self.rows = []
 
@@ -211,13 +215,16 @@ class Table:
         row_info = list(self._process_rowpans_entries())
 
         # actually render the cells
+        repeat_headings = (
+            self._repeat_headings is TableHeadingsDisplay.ON_TOP_OF_EVERY_PAGE
+        )
         self._fpdf.y += self._outer_border_margin[1]
         for i, row in enumerate(self.rows):
             # pylint: disable=protected-access
             page_break = self._fpdf._perform_page_break_if_need_be(
                 row_info[i].pagebreak_height
             )
-            if page_break and i >= self._num_heading_rows:
+            if page_break and repeat_headings and i >= self._num_heading_rows:
                 # repeat headings on top:
                 self._fpdf.y += self._outer_border_margin[1]
                 for row_idx in range(self._num_heading_rows):
