@@ -6,6 +6,8 @@ import pytest
 from fpdf import FPDF, FPDFException
 from fpdf.drawing import DeviceRGB
 from fpdf.fonts import FontFace
+from fpdf.table import TableCellFillMode
+
 from test.conftest import assert_pdf_equal, LOREM_IPSUM
 
 
@@ -267,6 +269,27 @@ def test_table_with_cell_fill(tmp_path):
     assert_pdf_equal(pdf, HERE / "table_with_cell_fill.pdf", tmp_path)
 
 
+class EvenOddCellFillMode:
+    @staticmethod
+    def should_fill_cell(i, j):
+        return i % 2 and j % 2
+
+
+def test_table_with_cell_fill_custom_class(tmp_path):
+    pdf = FPDF()
+    pdf.add_page()
+    pdf.set_font("Times", size=16)
+    lightblue = (173, 216, 230)
+    with pdf.table(
+        cell_fill_color=lightblue, cell_fill_mode=EvenOddCellFillMode()
+    ) as table:
+        for data_row in TABLE_DATA:
+            row = table.row()
+            for datum in data_row:
+                row.cell(datum)
+    assert_pdf_equal(pdf, HERE / "table_with_cell_fill_custom_class.pdf", tmp_path)
+
+
 def test_table_with_internal_layout(tmp_path):
     pdf = FPDF()
     pdf.add_page()
@@ -329,6 +352,33 @@ def test_table_with_single_top_line_layout_and_page_break(tmp_path):  # PR #912
     )
 
 
+def test_table_with_page_break_and_headings_repeated(tmp_path):  # issue 1151
+    pdf = FPDF()
+    pdf.add_page()
+    pdf.set_font("Times", size=16)
+    pdf.cell(text="repeat_headings=1:", new_y="NEXT")
+    with pdf.table(
+        MULTILINE_TABLE_DATA,
+        repeat_headings=1,
+    ):
+        pass
+    pdf.cell(text='repeat_headings="NONE":', new_y="NEXT")
+    with pdf.table(
+        MULTILINE_TABLE_DATA,
+        repeat_headings="NONE",
+    ):
+        pass
+    pdf.cell(text='repeat_headings="ON_TOP_OF_EVERY_PAGE":', new_y="NEXT")
+    with pdf.table(
+        MULTILINE_TABLE_DATA,
+        repeat_headings="ON_TOP_OF_EVERY_PAGE",
+    ):
+        pass
+    assert_pdf_equal(
+        pdf, HERE / "table_with_page_break_and_headings_repeated.pdf", tmp_path
+    )
+
+
 def test_table_align(tmp_path):
     pdf = FPDF()
     pdf.add_page()
@@ -732,3 +782,23 @@ def test_table_with_varying_col_count(tmp_path):
             table.row(subset)
 
     assert_pdf_equal(pdf, HERE / "table_with_varying_col_count.pdf", tmp_path)
+
+
+def test_table_cell_fill_mode(tmp_path):
+    pdf = FPDF()
+    pdf.add_page()
+    pdf.set_font("Helvetica")
+    light_sky_blue = (150, 200, 255)
+    headings_style = FontFace(fill_color=128)  # grey
+    for mode in ("ROWS", "COLUMNS", "EVEN_ROWS", "EVEN_COLUMNS"):
+        pdf.cell(text=f"cell_fill_mode=TableCellFillMode.{mode}:")
+        pdf.ln(10)
+        with pdf.table(
+            TABLE_DATA,
+            headings_style=headings_style,
+            cell_fill_mode=getattr(TableCellFillMode, mode),
+            cell_fill_color=light_sky_blue,
+        ):
+            pass
+        pdf.ln()
+    assert_pdf_equal(pdf, HERE / "table_cell_fill_mode.pdf", tmp_path)
